
#include <stdio.h>
#include <stdlib.h> 

#include <propeller.h>
#define P2_TARGET_MHZ 160
#include "sys/p2es_clock.h"

#define L_PIN 6
#define R_PIN 7
#define SYSTEM_CLOCK (_CLOCKFREQ)
#define SAMPLE_RATE 16000
#define DAC_MODE             (3 << 1) // DAC 16-bit dither, PWM
#define DIR_MODE             1        // Output enabled, overrides DIR
#define INIT_8BIT_DAC_VALUE  128
#define ANALOG_OUT_MODE      0x17     //  75 ohm, 2.0V DAC mode
#define SMARTPIN_DAC_MODE    (ANALOG_OUT_MODE << 16) | (INIT_8BIT_DAC_VALUE << 8) | (DIR_MODE << 6) | DAC_MODE
#define SAMPLE_PERIOD        SYSTEM_CLOCK  / SAMPLE_RATE  // CPU cycles between sample updates

#define DECAY_RATE           2

short pluck (int frq, unsigned int duration_ms);
void ex_1 (void);
void ex_2 (void);

void audio_init()
{

    __asm {
              wrpin   ##SMARTPIN_DAC_MODE, #L_PIN           // Config smartpin DAC mode on "left" pin
              wrpin   ##SMARTPIN_DAC_MODE, #R_PIN           // Config smartpin DAC mode on "right" pin
              wxpin   ##SAMPLE_PERIOD, #L_PIN               // Set sample period for left audio channel
              wxpin   ##SAMPLE_PERIOD, #R_PIN               // Set sample period for right audio channel
              dirh    #L_PIN                                // Enable smartpin DAC mode on left pin
              dirh    #R_PIN                                // Enable smartpin DAC mode on right pin
              setse1  #(1 << 6) | L_PIN                     // Event triggered every new sample period (when "left in pin rises")

    };

}



int main (void)
{
	clkset(_SETFREQ, _CLOCKFREQ);
	audio_init();
	
    ex_2(); 
    ex_1();
    pluck(440,0);
   
   return 0;
}

short pluck (int frq, unsigned int duration_ms)
{
  frq = 268435 * frq;

  //unsigned int start;
  //unsigned int stop ;
  int start = getcnt();
  int phi = 0;
  int sample;
  unsigned int tick=0;
  unsigned int amplitude = (1 << 28);
  unsigned int decay = (1 << 31) - (DECAY_RATE << 17);
  unsigned int temp;
#if 0
  __asm{
	  getct		start
	  add		start,duration_ms
	  mov		stop,start
	  
	};
#endif
   if(duration_ms == 0)
   {	
		while( 1 )
		{
			__asm {
                        mov     temp, amplitude
                        shr     temp, #16
			qrotate	temp, phi
            getqy		sample
            qmul        amplitude, decay
            getqy       amplitude
            shl         amplitude, #1
            xor     	sample, ##$8000
            wypin   	sample, #L_PIN      
            waitse1
            add			phi,frq
			};
		}
   }

   else
   {
                int duration_cycles = 160000 * duration_ms;
                start = getcnt();
		while( getcnt() - start < duration_cycles)
		{
			__asm {
                        mov     temp, amplitude
                        shr     temp, #16
			qrotate	temp, phi
            getqy		sample
            qmul        amplitude, decay
            getqy       amplitude
            shl         amplitude, #1
            xor     	sample, ##$8000
            wypin   	sample, #L_PIN      
            waitse1
            add			phi,frq
            getct		tick 
			};
		}
	}
	return 1;
}


void ex_1 (void)
{
  
  pluck (330, 330);
  pluck (330, 330);
  pluck (330, 660);

  pluck (330, 330);
  pluck (330, 330);
  pluck (330, 660);


  pluck (330, 330);
  pluck (392, 330);
  pluck (262, 330);
  pluck (294, 330);
  pluck (330, 660);

  pluck (349, 330);
  pluck (349, 330);
  pluck (349, 330);

  pluck (349, 330);
  pluck (349, 330);
  pluck (330, 330);

  pluck (330, 330);
  pluck (330, 330);

  pluck (330, 330);
  pluck (294, 330);
  pluck (294, 330);
  pluck (330, 330);

  pluck (294, 660);
  pluck (392, 660);

  pluck (330, 330);
  pluck (330, 330);
  pluck (330, 660);

  pluck (330, 330);
  pluck (330, 330);
  pluck (330, 660);


  pluck (330, 330);
  pluck (392, 330);
  pluck (262, 330);
  pluck (294, 330);
  pluck (330, 660);

  pluck (349, 330);
  pluck (349, 330);
  pluck (349, 330);

  pluck (349, 330);
  pluck (349, 330);
  pluck (330, 330);

  pluck (330, 330);
  pluck (330, 330);

  pluck (330, 330);
  pluck (294, 330);
  pluck (294, 330);
  pluck (330, 330);

  pluck (294, 800);
}

void ex_2 (void)
{
  pluck (262,500);
  pluck (294,500);
  pluck (330,500);
  pluck (350,500);
  pluck (393,500);
  pluck (440,500);
  pluck (495,500);
  pluck (524,500);
}
